@@ -14,6 +14,7 @@
 
 import copy
 import tempfile
+import time
 
 import mock
 import six
@@ -38,6 +39,7 @@ NODE1 = {'id': 123,
          'driver_info': {'user': 'foo', 'password': 'bar'},
          'properties': {'num_cpu': 4},
          'name': 'fake-node-1',
+         'resource_class': 'foo',
          'extra': {}}
 NODE2 = {'id': 456,
          'uuid': '66666666-7777-8888-9999-111111111111',
@@ -47,13 +49,13 @@ NODE2 = {'id': 456,
          'driver': 'fake too',
          'driver_info': {'user': 'foo', 'password': 'bar'},
          'properties': {'num_cpu': 4},
+         'resource_class': 'bar',
          'extra': {}}
 PORT = {'id': 456,
         'uuid': '11111111-2222-3333-4444-555555555555',
         'node_id': 123,
         'address': 'AA:AA:AA:AA:AA:AA',
         'extra': {}}
-
 POWER_STATE = {'power_state': 'power off',
                'target_power_state': 'power on'}
 
@@ -172,6 +174,20 @@ fake_responses = {
             {"nodes": [NODE1]},
         )
     },
+    '/v1/nodes/?resource_class=foo':
+    {
+        'GET': (
+            {},
+            {"nodes": [NODE1]},
+        )
+    },
+    '/v1/nodes/?chassis_uuid=%s' % NODE2['chassis_uuid']:
+    {
+        'GET': (
+            {},
+            {"nodes": [NODE2]},
+        )
+    },
     '/v1/nodes/detail?instance_uuid=%s' % NODE2['instance_uuid']:
     {
         'GET': (
@@ -549,6 +565,25 @@ class NodeManagerTest(testtools.TestCase):
         self.assertThat(nodes, HasLength(1))
         self.assertEqual(NODE1['uuid'], getattr(nodes[0], 'uuid'))
 
+    def test_node_list_resource_class(self):
+        nodes = self.mgr.list(resource_class="foo")
+        expect = [
+            ('GET', '/v1/nodes/?resource_class=foo', {}, None),
+        ]
+        self.assertEqual(expect, self.api.calls)
+        self.assertThat(nodes, HasLength(1))
+        self.assertEqual(NODE1['uuid'], getattr(nodes[0], 'uuid'))
+
+    def test_node_list_chassis(self):
+        ch2 = NODE2['chassis_uuid']
+        nodes = self.mgr.list(chassis=ch2)
+        expect = [
+            ('GET', '/v1/nodes/?chassis_uuid=%s' % ch2, {}, None),
+        ]
+        self.assertEqual(expect, self.api.calls)
+        self.assertThat(nodes, HasLength(1))
+        self.assertEqual(NODE2['uuid'], getattr(nodes[0], 'uuid'))
+
     def test_node_list_no_maintenance(self):
         nodes = self.mgr.list(maintenance=False)
         expect = [
@@ -1032,3 +1067,117 @@ class NodeManagerTest(testtools.TestCase):
         ]
         self.assertEqual(expect, self.api.calls)
         self.assertEqual(NODE_VENDOR_PASSTHRU_METHOD, vendor_methods)
+
+    def _fake_node_for_wait(self, state, error=None, target=None):
+        spec = ['provision_state', 'last_error', 'target_provision_state']
+        return mock.Mock(provision_state=state,
+                         last_error=error,
+                         target_provision_state=target,
+                         spec=spec)
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(node.NodeManager, 'get', autospec=True)
+    def test_wait_for_provision_state(self, mock_get, mock_sleep):
+        mock_get.side_effect = [
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('active')
+        ]
+
+        self.mgr.wait_for_provision_state('node', 'active')
+
+        mock_get.assert_called_with(self.mgr, 'node')
+        self.assertEqual(3, mock_get.call_count)
+        mock_sleep.assert_called_with(node._DEFAULT_POLL_INTERVAL)
+        self.assertEqual(2, mock_sleep.call_count)
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(node.NodeManager, 'get', autospec=True)
+    def test_wait_for_provision_state_timeout(self, mock_get, mock_sleep):
+        mock_get.return_value = self._fake_node_for_wait(
+            'deploying', target='active')
+
+        self.assertRaises(exc.StateTransitionTimeout,
+                          self.mgr.wait_for_provision_state, 'node', 'active',
+                          timeout=0.001)
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(node.NodeManager, 'get', autospec=True)
+    def test_wait_for_provision_state_error(self, mock_get, mock_sleep):
+        mock_get.side_effect = [
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('deploy failed', error='boom'),
+        ]
+
+        self.assertRaisesRegexp(exc.StateTransitionFailed,
+                                'boom',
+                                self.mgr.wait_for_provision_state,
+                                'node', 'active')
+
+        mock_get.assert_called_with(self.mgr, 'node')
+        self.assertEqual(2, mock_get.call_count)
+        mock_sleep.assert_called_with(node._DEFAULT_POLL_INTERVAL)
+        self.assertEqual(1, mock_sleep.call_count)
+
+    @mock.patch.object(node.NodeManager, 'get', autospec=True)
+    def test_wait_for_provision_state_custom_delay(self, mock_get):
+        mock_get.side_effect = [
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('active')
+        ]
+
+        delay_mock = mock.Mock()
+        self.mgr.wait_for_provision_state('node', 'active',
+                                          poll_delay_function=delay_mock)
+
+        mock_get.assert_called_with(self.mgr, 'node')
+        self.assertEqual(2, mock_get.call_count)
+        delay_mock.assert_called_with(node._DEFAULT_POLL_INTERVAL)
+        self.assertEqual(1, delay_mock.call_count)
+
+    def test_wait_for_provision_state_wrong_input(self):
+        self.assertRaises(ValueError, self.mgr.wait_for_provision_state,
+                          'node', 'active', timeout='42')
+        self.assertRaises(ValueError, self.mgr.wait_for_provision_state,
+                          'node', 'active', timeout=-1)
+        self.assertRaises(TypeError, self.mgr.wait_for_provision_state,
+                          'node', 'active', poll_delay_function={})
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(node.NodeManager, 'get', autospec=True)
+    def test_wait_for_provision_state_unexpected_stable_state(
+            self, mock_get, mock_sleep):
+        # This simulates aborted deployment
+        mock_get.side_effect = [
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('available'),
+        ]
+
+        self.assertRaisesRegexp(exc.StateTransitionFailed,
+                                'available',
+                                self.mgr.wait_for_provision_state,
+                                'node', 'active')
+
+        mock_get.assert_called_with(self.mgr, 'node')
+        self.assertEqual(2, mock_get.call_count)
+        mock_sleep.assert_called_with(node._DEFAULT_POLL_INTERVAL)
+        self.assertEqual(1, mock_sleep.call_count)
+
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(node.NodeManager, 'get', autospec=True)
+    def test_wait_for_provision_state_unexpected_stable_state_allowed(
+            self, mock_get, mock_sleep):
+        mock_get.side_effect = [
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('available'),
+            self._fake_node_for_wait('deploying', target='active'),
+            self._fake_node_for_wait('active'),
+        ]
+
+        self.mgr.wait_for_provision_state('node', 'active',
+                                          fail_on_unexpected_state=False)
+
+        mock_get.assert_called_with(self.mgr, 'node')
+        self.assertEqual(4, mock_get.call_count)
+        mock_sleep.assert_called_with(node._DEFAULT_POLL_INTERVAL)
+        self.assertEqual(3, mock_sleep.call_count)
